/*
 * simplistic geometrical linux logo wallpaper
 *
 * design and scene description file created by Holger Bettag
 * (hobold@vectorizer.org) in Apr-2019
 *
 * feel free to use and modify logo and scene description file as you
 * wish, but please give credit where credit is due. thanks!
 *
 * refer to http://www.povray.org to learn about the open source
 * Persistence of Vision ray tracer
 *
 *
 * when rendered as an animation, you get a cyclical loop of slowly dancing
 * shadows all across the image. when rendered as a single frame, you get a
 * fairly clear shot of the logo for use as a desktop wallpaper.
 */

#global_settings{
  max_trace_level 10
  assumed_gamma 1.0

  radiosity {
    count 200            // a bit on the high side to reduce noise
    error_bound 0.0035   // a bit on the low side to reduce noise
    recursion_limit 2
  }
}


camera {
  // the logo really is a pure 2D design, so
  // opt out of perspective foreshortening
  orthographic
  location  <0, 0, 30.0>
  direction <0, 0, -2>
  up        12.5*<0, 1, 0>
  right     12.5*<16/9, 0, 0>
  sky       <0,1,0>
  look_at   <0, 0, 0>
}


#default {
  texture {
    pigment { color rgb 0.5 }
    finish {
      ambient 0.0
      diffuse 0.6
      specular 0.2
    }
  }
}



// letter bevel distance from outer rim; from > 0.0 to <= 1.0
#declare inward = 1;


// letter colors (basically a pastel rainbow with minor tweaks)
#declare colorL = color rgb < 0.84, 0.49, 0.99 >;
#declare colorI = color rgb < 0.99, 0.49, 0.49 >;
#declare colorIdot = color rgb < 0.99, 0.89, 0.49 >;
#declare colorN = color rgb < 0.49, 0.91, 0.49 >;
#declare colorU = color rgb < 0.49, 0.82, 0.91 >;
#declare colorX = color rgb < 0.43, 0.43, 0.99 >;


#declare LetterL = difference {
  box { < -9, -4, 0 >, < -3, 4, 1 > }
  intersection { // convex (inner) corner
    plane { < -1, 0, -inward >, 0
      translate -7*x
    }
    plane { < 0, -1, -inward >, 0
      translate -2*y
    }
  }
  plane { < 0, -1, -inward >, 0  // top
    translate 4*y
  }
  plane { < 1, 0, -inward >, 0  // left
    translate -9*x
  }
  plane { < 0, 1, -inward >, 0  // bottom
    translate -4*y
  }
  plane { < -1, 0, -inward >, 0  // right
    translate -3*x
  }
  pigment { color colorL }
}

#declare LetterI = difference {
  box { < -5, -2, 0 >, < -3, 2, 1 > }
  plane { < 0, -1, -inward >, 0  // top
    translate 2*y
  }
  plane { < 1, 0, -inward >, 0  // left
    translate -5*x
  }
  plane { < 0, 1, -inward >, 0  // bottom
    translate -2*y
  }
  plane { < -1, 0, -inward >, 0  // right
    translate -3*x
  }
  pigment { color colorI }
}

#declare LetterIdot = intersection {
  cone { < 0, 0, -1 >, 1 + inward, < 0, 0, 1 >, max(1 - inward, 0) }
  plane { -z, 0 }
  pigment { color colorIdot }
  translate < -4, 3, 0 >
}

#declare LetterN = difference {
  union {
    difference {  // horizontal top block
      box { < -4, 2, 0 >, < -2, 4, 1 > }
      plane { < 0, -1, -inward >, 0  // top
        translate 4*y
      }
      plane { < 0, 1, -inward >, 0  // bottom
        translate 2*y
      }
    }
    difference {  // vertical right block
      box { < -1, -4, 0 >, < 1, 1, 1 > }
      plane { < 1, 0, -inward >, 0  // left
        translate -x
      }
      plane { < -1, 0, -inward >, 0  // right
        translate x
      }
    }
    intersection {  // outer curve
      cone { < -2, 1, 0 >, 3, < -2, 1, 2 >, 3 - 2*inward }
      box { < -2, 1, 0 >, < 1, 4, 1 > }
    }
  }
  cone { < -2, 1, -1 >, 1 - inward, < -2, 1, 2 >, 1 + 2*inward }  // inner curve
  cone { < -4, 3, -1 >, 1 - inward, < -4, 3, 2 >,1 + 2*inward }  // horiz left
  plane { < 0, 1, -inward >, 0  // bottom
    translate -4*y
  }
  pigment { color colorN }
}

#declare LetterU = union {
  difference {  // left vertical, top
    box { < -1, 1, 0 >, < 1, 4, 1 > }
    plane { < 0, -1, -inward >, 0  // top
      translate 4*y
    }
    plane { < 1, 0, -inward >, 0  // left
      translate -x
    }
    plane { < -1, 0, -inward >, 0  // right
      translate x
    }
    // outer N curve
    cone { < -2, 1, -1 >, 3 - inward, < -2, 1, 2 >, 3 + 2*inward }
  }
  difference {  // lower U curve
    cone { < 2, -1, 0 >, 3, < 2, -1, 1 >, 3 - inward }  // outer curve
    plane { -y, 0  // remove upper half
      translate -y
    }
    // inner curve
    cone { < 2, -1, -1 >, 1 - inward, < 2, -1, 2 >, 1 + 2*inward }
    plane { < 1, 0, -inward >, 0  // left
      translate x
    }
  }
  difference {  // right vertical
    box { < 3, -1, 0 >, < 5, 4, 1 > }
    plane { < 0, -1, -inward >, 0  // top
      translate 4*y
    }
    plane { < 1, 0, -inward >, 0  // left
      translate 3*x
    }
    plane { < -1, 0, -inward >, 0  // right
      translate 5*x
    }
  }
  pigment { color colorU }
}

#declare LetterX = union {
  difference {
    union {
      box { < 7, 2, 0 >, < 9, 4, 1 > }  // top right
      box { < 3, -4, 0 >, < 5, -2, 1 > }  // bottom left
      box { < 7, -4, 0 >, < 9, -2, 1 > }  // bottom right
    }
    plane { < 0, -1, -inward >, 0  // top
      translate 4*y
    }
    plane { < 0, 1, -inward >, 0  // bottom
      translate -4*y
    }
    plane { < 1, 0, -inward >, 0  // outer left
      translate 3*x
    }
    plane { < -1, 0, -inward >, 0  // outer right
      translate 9*x
    }
    intersection {  // inner vertical bevels
      plane { < 1, 0, -inward >, 0  // inner left
        translate 7*x
      }
      plane { < -1, 0, -inward >, 0  // inner right
        translate 5*x
      }
    }
    // outer U curve
    cone { < 2, -1, -1 >, 3 - inward, < 2, -1, 2 >, 3 + 2*inward }
  }
  difference {  // middle curved part
    union {
      cone { < 6, 2, 0 >, 3, < 6, 2, 1 >, 3 - inward }  // upper curve
      cone { < 6, -2, 0 >, 3, < 6, -2, 1 >, 3 - inward }  // lower curve
    }
    plane { -y, 0  // remove upper part
      translate 2*y
    }
    plane { y, 0  // remove upper part
      translate -2*y
    }
    // outer U curve
    cone { < 2, -1, -1 >, 3 - inward, < 2, -1, 2 >, 3 + 2*inward }
    cone {<6,2,-1>,1-inward,<6,2,2>,1 + 2*inward}  // inner curve top
    cone {<6,-2,-1>,1-inward,<6,-2,2>,1 + 2*inward}  // inner curve top
    intersection {  // left vertical top left bevel
      plane { < 1, 0, -inward >, 0  // outer left
        translate 5*x
      }
      plane { -y, 0
        translate -y
      }
    }
  }
  pigment { color colorX }
}

// phase shift local clock a bit such that first frame of cycle is beauty shot
#declare klock = clock + 7/40;

#declare Frame = union {
  object { LetterL
    scale < 1, 1, 1.5*inward >
    translate 0.5*(sin((-klock + 0/6)*2*pi) + 1)*z
  }
  object { LetterI
    scale < 1, 1, 1.5*inward >
    translate 0.5*(sin((-klock + 1/6)*2*pi) + 1)*z
  }
  object { LetterIdot
    scale < 1, 1, 1.5*inward >
    translate 0.5*(sin((-klock + 2/6)*2*pi) + 1)*z
  }
  object { LetterN
    scale < 1, 1, 1.5*inward >
    translate 0.5*(sin((-klock + 3/6)*2*pi) + 1)*z
  }
  object { LetterU
    scale < 1, 1, 1.5*inward >
    translate 0.5*(sin((-klock + 4/6)*2*pi) + 1)*z
  }
  object { LetterX
    scale < 1, 1, 1.5*inward >
    translate 0.5*(sin((-klock + 5/6)*2*pi) + 1)*z
  }
}

// instantiate all visible objects for rendering
union {
  object { Frame }  // the combined letters

  // wall behind letters
  plane { z, 0
    pigment { color rgb <0.8, 0.79, 0.78> }  // image background color
  }
  
  rotate 4*z  // slightly tilt logo to accomodate 16:9 aspect ratio
}

// area lights for softer shadows & subtle color shifts
light_source { <-25, 25, 25> color rgb 0.4*<1.0, 0.8, 0.6>
  area_light <14, 14, 0>, <14, -14, 0>, 17, 17
  adaptive 2
  jitter
  rotate 2*360*(125/400)*z
}
light_source { <105, 35, 35> color rgb 0.3*<0.6, 1.0, 0.8>
  area_light <0, 14, 0>, <0, 0, 14>, 17, 17
  adaptive 2
  jitter
  rotate 360*(125/400)*z
}

// this does NOT control the image background here, but the color and
// intensity of the environmental light
background { color rgb 0.02 }

